REBOL []

make root-protocol [
	scheme: 'rmc
    port-id: 4000
	port-flags: system/standard/port-flags/pass-thru or 32 ; /binary
	
	header: make binary! 4
	len: none
	pending?: no
	last-cmd: url: call-line: none
	auto-import?: no
	func-nb: none
	show-help?: true
	
	sys-copy: 	get in system/words 'copy
	sys-insert: get in system/words 'insert
	sys-close: 	get in system/words 'close
	net-log: 	get in net-utils	'net-log
	
	read: func [[throw] port data size][
		if -1 = len: read-io port/sub-port data size [
			sys-close port/sub-port
			;throw throws/closed
		]
		net-log ["low level read of " len "bytes"] 
		len
    ]
    
	defrag-read: func [port [port!] buf [binary!] expected [integer!]][
		clear buf
		while [expected > length? buf][
			read port buf expected - length? buf
		]
	]
	
	read-packet: func [port /local bytes body][
		clear header
		defrag-read port header 4
		bytes: to-integer header
		body: make binary! bytes + 16
		defrag-read port body bytes
		pending?: no
		load to-string body
	]
	
	validate?: func [data][
		if not find [import do] last-cmd: data/1 [
			last-cmd: none
			make error! "Invalid RMC command!"
		]
		true
	]
	
	proto-local: [/local p res]
	
	proto-body: [
		insert p: open (url) reduce ['do (call-line)]
		res: copy p close p	res
	]
	
	extract-args: func [blk][
		while [not tail? blk][
			either word? first blk [blk: next blk][remove blk]
		]
		head blk
	]
	
	build-funcs: func [port specs /local body][
		func-nb: divide length? specs 2
		if show-help? [print "^/Import list:^/-----------"]
		foreach [name spec] specs [
			if show-help? [
				print reform [
					uppercase mold name "-"
					either string? first spec [first spec]["no description"]
				]
			]
			url: join rmc:// port/host
			call-line: head sys-insert extract-args sys-copy spec to-lit-word :name
			body: compose/deep proto-body
			set :name func append spec proto-local bind body 'system
		]
	]
	
	process-response: func [port [port!] /local body][
		body: read-packet port
		either body/1 = 'ok [
			either last-cmd = 'import [build-funcs port body/2][body/2]
		][
			body: first reduce next body
			make error! either string? body [body][
				reduce remove/part bind first body in body 'self 2
			]
		]
		
	]
	
;------ Public interface ------

    open: func [port [port!]][
        open-proto port    
        port/sub-port/state/flags: 524835	; force /direct/binary mode
        if port/target = "import" [
        	auto-import?: on
        	insert port [import all]
        ]
    ]
	
	insert: func [[throw] port [port!] blk [block!]][
		if validate? blk [
			blk: mold blk
			sys-insert blk debase/base to-hex length? blk 16
			write-io port/sub-port blk length? blk
			pending?: yes
		]
		if auto-import? [auto-import?: no]
	]

	copy: func [port /local res][
		either pending? [
			pending?: no
			res: process-response port
			either last-cmd = 'do [res][func-nb]
		][none]
	]

	;--- Register ourselves. 
	net-utils/net-install RMC self port-id
]